extensions [ time table csv ]

__includes [ "parameters.nls" ]

breed [ nodes node ]        ; Points at which we have input

globals
[
  ; Global variables calculated by code
  cells                  ; An agentset of cells (patches that are modeled as habitat)
  time                   ; A Logo-time value for time at which current time step started
  formatted-time         ; A string with current time
  next-time              ; The time at which the current time step ends
  step-length            ; Length of the current time step (days)
  prev-step-length       ; Length of the previous time step (days)
  run-duration           ; Maximum number of days to simulate (days)
  end-time-code          ; Logo-time variable for ending time
  run-time               ; Execution time (sec)
  max-cell-elevation     ;
  min-cell-elevation
  flow                   ; River flow
  temperature            ; Water temperature

  ; Internal data structures
  input-time-series  ; Time-series input set (time extension's LogoTime data type)
  input-time-list    ; List of times in input-time-series
  depth-flow-list    ; List of flows in the depth hydraulic input file
  velocity-flow-list ; List of flows in the velocity hydraulic input file
  node-patches       ; A *table* linking cell numbers to corresponding patches

  ; =============== Parameters ===============
  ; Definitions and values of the following parameters are in "parameters.nls"

  ; Resolution parameters
  patch-width           ; Size (width, m) of NetLogo patches
  max-node-distance     ; Maximum distance (m) a patch can be from an input node and still be a cell.

  ; File names
  geom-file-name
  cell-file-name
  velocity-file-name
  depth-file-name
  time-series-file-name

  ; Display parameters
  max-shade-depth       ; The depth at which cell colors are bluest (m)
  max-shade-velocity    ; The velocity at which cell colors are reddest (m/s)
  write-frames?         ; Set to true to write the View to a graphics file each tick, for making movies of model run

]

nodes-own
[
  nodes-node-number       ; Arbitrary reference number for cells
  nodes-elevation    ; Cell bed elevation
  nodes-depth-lookup  ; Lookup table of depth values for each flow in global depth-flow-list
  nodes-velocity-lookup  ; Lookup table of vel values for each flow in global velocity-flow-list
  nodes-flow-at-wetting  ; Flow above which depth becomes non-zero

  nodes-actual-x    ; Cell coordinates from geometry input file, for output
  nodes-actual-y
]

patches-own
[
  node-number       ; Reference number for the node that patch gets variables from
  depth             ; Water depth in METERS
  velocity          ; Water velocity in m/s

  elevation    ; Cell bed elevation
  cell-suitability ; An example cell habitat variable
  depth-lookup  ; Lookup table of depth values for each flow in global depth-flow-list
  velocity-lookup  ; Lookup table of vel values for each flow in global velocity-flow-list
  flow-at-wetting  ; Flow above which depth becomes non-zero

  actual-x    ; Cell coordinates from geometry input file, for output
  actual-y
]

turtles-own
[
]

to setup

  ca
  reset-ticks

  show "Setting parameter values"
  set-parameters  ; This procedure is in the separate "parameters.nls" file.

  show "Reading geometry"
  read-geom
  display

  show "Reading cell variables"
  read-cell-variables

  show "Reading hydraulics and building lookup tables"
  read-hydraulics

  file-close ; Just in case a file got left open.

  ; Set time variables
  show "Setting time variables"
  set time time:create-with-format start-time "M/d/yyyy H:mm"
  set formatted-time time:show time "MM/dd/yyyy HH:mm"
  set end-time-code time:plus time run-duration "DAYS"
  if time:is-after? time end-time-code [ error "Really, the simulation must end after it starts" ]
  output-print formatted-time

  ; Read all the input file's data into a LogoTimeSeries variable
  show "Reading time-series input file"
  if not file-exists? time-series-file-name [ error (word "Input file " time-series-file-name " not found") ]
  set input-time-series ts-load-with-format time-series-file-name "M/d/yyyy H:mm"

  ; Extract from the LogoTimeSeries a list of the time values in the input file
  set input-time-list ts-get-range input-time-series time end-time-code "LOGOTIME"
  if empty? input-time-list [ error "Time-series input file contains no data for start-time through run-duration" ]
  ; Test output
  ; foreach input-time-list [ a-time -> show time:show a-time "MMMM d, yyyy HH:mm" ]

  ; Initialize the next-time variable
  set next-time 0
  set step-length 0

  show "Updating habitat"
  update-habitat

  show "End of setup"

end

to reset
  ; This procedure finishes setting up AND allows the model to be re-started
  ; without re-building the habitat

  ; Make sure "setup" has been run at least once
  if geom-file-name = 0 [ error "You neglected to run setup!" ]

  ask turtles [ die ]
  clear-output
  clear-plot

  reset-ticks

  ; random-seed 111  ; Un-comment this to set random number seed.

  file-close ; Just in case a file got left open.

  ; Set time variables
  show "Setting time variables"
  set time time:create-with-format start-time "M/d/yyyy H:mm"
  set formatted-time time:show time "MM/dd/yyyy HH:mm"
  set end-time-code time:plus time run-duration "DAYS"
  if time:is-after? time end-time-code [ error "Really, the simulation must end after it starts" ]
  output-print formatted-time

  ; Read all the input file's data into a LogoTimeSeries variable
  show "Reading time-series input file"
  if not file-exists? time-series-file-name [ error (word "Input file " time-series-file-name " not found") ]
  set input-time-series ts-load-with-format time-series-file-name "M/d/yyyy H:mm"

  ; Extract from the LogoTimeSeries a list of the time values in the input file
  set input-time-list ts-get-range input-time-series time end-time-code "LOGOTIME"
  if empty? input-time-list [ error "Time-series input file contains no data for start-time through run-duration" ]
  ; Test output
  ; foreach input-time-list [ a-time -> show time:show a-time "MMMM d, yyyy HH:mm" ]

  ; Initialize the next-time variable
  set next-time 0
  set step-length 0

  show "Updating habitat"
  update-habitat

  show "End of reset"

end

to go

  if (ticks < 1) [ reset-timer ]

  tick  ; Advance the time step counter

  ; Determine length of the current time step: the time (days) between current time and
  ; the next time in the input file.
  ; Time values are all taken from the list of time values in the input file
  ; If it is not the first tick, advance the time by reading the first value on the time list
  set time time:copy first input-time-list
  set formatted-time time:show time "MM/dd/yyyy HH:mm"

  ; Then remove the current time from the list, and stop if it is the last time.
  set input-time-list remove-item 0 input-time-list
  if (empty? input-time-list)
  [
    set run-time timer
    show (word "Execution finished in " run-time " seconds")
    stop
  ]

  ; Now, the first item on the time list is the start of the *next* time step
  set next-time time:copy first input-time-list

  ; Finally, calculate the time step length, and remember length of previous step
  set prev-step-length step-length
  set step-length time:difference-between time next-time "days"

  ; Set daily depths, velocities
  update-habitat

  ; Display current time, flow, temperature
  output-print (word (time:show time "MMMdd, yyyy HH:mm") "; step length: " step-length "; flow: " flow " temperature: " temperature)

end

to read-geom  ; Global procedure to set spatial scale and assign cell numbers to patches

  show "Starting to read geometry"

  ; First, make sure file name and file exist
  if empty? geom-file-name [ error "Missing geometry file name" ]
  if not file-exists? geom-file-name
    [ user-message (word "Geometry file " geom-file-name " not found")
      stop
    ]

  ; Open file and skip the 3 header lines
  file-open geom-file-name
  let header-string "a string"
  repeat 3 [set header-string file-read-line ]

  ; Create lists of cell numbers, X and Y coordinates in file
  let node-numbers (list)
  let actual-x-coords (list)
  let actual-y-coords (list)

  while [not file-at-end?]
  [
   set node-numbers fput file-read node-numbers
   set actual-x-coords fput file-read actual-x-coords
   set actual-y-coords fput file-read actual-y-coords
  ]
  file-close

  ; Calculate space dimensions and patch size
  let num-x-coords length remove-duplicates actual-x-coords
  let num-y-coords length remove-duplicates actual-y-coords

  ; Single-cell-width spaces cause division by zero below
  if num-x-coords = 0 or num-y-coords = 0
    [ error "Sorry, one-cell-width spaces are not allowed" ]

  let min-actual-x min actual-x-coords
  let min-actual-y min actual-y-coords
  let max-actual-x max actual-x-coords
  let max-actual-y max actual-y-coords

;  let patch-size-x (max-actual-x - min-actual-x) / (num-x-coords - 1)
;  let patch-size-y (max-actual-y - min-actual-y) / (num-y-coords - 1)
;
;  ; Check for problems
;  if abs ((patch-size-x - patch-size-y) / patch-size-x) > 0.01
;  [ user-message "Cells in geometry file are not square" ]

  ; Set world size
  let maximum-patch-x round ((max-actual-x - min-actual-x) / patch-width)
  let maximum-patch-y round ((max-actual-y - min-actual-y) / patch-width)
  ; Approximate a good patch size - turned off
  ; set-patch-size max (list (21 - (maximum-patch-x * 0.051)) 0.5)
  resize-world 0 maximum-patch-x 0 maximum-patch-y

  ; Set patch variables to dummy values for key variables, to distinguish cell
  ; from non-cell patches. Code moved 7 May 2018
  ask patches
  [
    set node-number -999
    set depth -999
    set velocity -999
    set pcolor 52 ; Forest green
  ]

  show "Creating nodes"
  ; Create nodes, Set variable values
  let index 0
  while [ index < length node-numbers ]
  [
    let the-x item index actual-x-coords
    let transformed-x  ((the-x - min-actual-x) / patch-width)
    let the-y item index actual-y-coords
    let transformed-y  ((the-y - min-actual-y) / patch-width)
    let the-cell-num item index node-numbers

    ; Check for illegal cell numbers
    if the-cell-num <= 0 [ error "Error in cell geometry: Non-positive cell number" ]

    create-nodes 1
    [
      set nodes-node-number the-cell-num      ; Arbitrary reference number for cells
      set nodes-actual-x the-x   ; Cell coordinates from geometry input file, for output
      set nodes-actual-y the-y
      setxy transformed-x transformed-y
    ]

;    ask patch transformed-x transformed-y
;    [
;      set node-number the-cell-num
;      set actual-x the-x
;      set actual-y the-y
;      set pcolor node-number
;    ]

 ;   table:put node-patches the-cell-num (patch transformed-x transformed-y)
    set index index + 1
  ]

  show "Setting nodes of patches"
  ; Create the table linking cell numbers to patches
  set node-patches table:make
  ask patches ; set the node number of patches and the patch-sets of nodes
  [
    let near-nodes nodes in-radius (max-node-distance / patch-width)
    if any? near-nodes
    [
      let patches-node min-one-of near-nodes [distance myself]
      set node-number [nodes-node-number] of patches-node
      set actual-x [nodes-actual-x] of patches-node
      set actual-y [nodes-actual-y] of patches-node
      ifelse table:has-key? node-patches node-number
      [
        let the-patch-set table:get node-patches node-number
        set the-patch-set (patch-set the-patch-set self)
        table:put node-patches node-number the-patch-set
      ]
      [
        table:put node-patches node-number (patch-set self)
      ]
    ]
  ]

  ; We're done with nodes, so delete them
  ask nodes [die]

  ; Finally, create an agentset of patches that are modeled cells
  set cells patches with [node-number > 0]

  show "Done reading cell geometry"
  show (word "Input contained " (count nodes) " habitat nodes")

end

to read-cell-variables  ; Observer procedure to read in cell data

  ; First, make sure file name and file exist
  if empty? cell-file-name [ error "Missing cell file name" ]
  if not file-exists? cell-file-name
    [ user-message (word "Cell variables file " cell-file-name " not found")
      stop
    ]

  ; Open file and skip the 3 header lines
  file-open cell-file-name
  let header-string "a string"
  repeat 3 [set header-string file-read-line ]

  ; Read in cell variables
  let a-node-number 0
  let an-elevation 0
  let a-suitability 0
  let a-has-shelter 0

  while [not file-at-end?]
  [
   set a-node-number file-read
   set an-elevation file-read
   set a-suitability file-read
   ; see if this node is in table; it may not be if no patches use it
   if table:has-key? node-patches a-node-number
   [
    ask (table:get node-patches a-node-number)
    [
      set elevation an-elevation
      set cell-suitability a-suitability
     ] ; ask (table:get
    ] ; if table:has-key?
  ]

  file-close

  ; Now set globals based on cell variables
  set max-cell-elevation max [elevation] of cells
  set min-cell-elevation min [elevation] of cells

end

to read-hydraulics
  ; Observer procedure to read in depth and velocity lookup tables for each cell.

  ; Depth first

  ; First, make sure file name and file exist
  if empty? depth-file-name [ error "Missing depth file name" ]
  if not file-exists? depth-file-name
    [ error (word "Depth file " depth-file-name " not found") ]

  ; Open file and skip the 3 header lines
  file-open depth-file-name
  let header-string "a string"
  repeat 3 [set header-string file-read-line ]

  ; Read in the number of flows
  let num-depth-flows file-read
  if not is-number? num-depth-flows [ error "Error reading number of flows in depth file" ]

  ; Read in the flow list
  set depth-flow-list (list)
  repeat num-depth-flows [ set depth-flow-list lput file-read depth-flow-list ]
  ; show depth-flow-list

  ; Read in the depth table for each cell
  while [not file-at-end?]
  [
    let next-cell file-read
    let a-depth-list (list)
    repeat num-depth-flows [ set a-depth-list lput file-read a-depth-list ]
    if table:has-key? node-patches next-cell  ; Check whether node is used
    [
     ask (table:get node-patches next-cell)
     [  set depth-lookup a-depth-list
     ]
    ]

  ]
  file-close

  ; Velocity second

  ; First, make sure file name and file exist
  if empty? velocity-file-name [ error "Missing velocity file name" ]
  if not file-exists? velocity-file-name
    [ error (word "Velocity file " velocity-file-name " not found") ]

  ; Open file and skip the 3 header lines
  file-open velocity-file-name
  repeat 3 [set header-string file-read-line ]

  ; Read in the number of flows
  let num-vel-flows file-read
  if not is-number? num-vel-flows [ error "Error reading number of flows in velocity file" ]

  ; Read in the flow list
  set velocity-flow-list (list)
  repeat num-vel-flows [ set velocity-flow-list lput file-read velocity-flow-list ]
  ; show velocity-flow-list

  ; Read in the velocity table for each cell
  while [not file-at-end?]
  [
    let next-cell file-read
    let a-vel-list (list)
    repeat num-vel-flows [ set a-vel-list lput file-read a-vel-list ]
    if table:has-key? node-patches next-cell  ; Check whether node is used
    [
      ask (table:get node-patches next-cell)
      [
        set velocity-lookup a-vel-list
        ; show velocity-lookup
      ]
    ]

  ]
  file-close

  ; Some error checking
  ask cells
  [
    if length depth-lookup != num-depth-flows
    [ error (word "Cell " node-number "does not have correct number of depths in lookup table") ]
    if length velocity-lookup != num-vel-flows
    [ error (word "Cell " node-number "does not have correct number of velocities in lookup table") ]
  ]

  ; Now set the minimum flow at which cell is wet
  ask cells
  [
    ifelse first depth-lookup > 0.0  ; Depth is non-zero at lowest lookup table flow
    [
      set flow-at-wetting 0.0
    ]
    [ ; Depth is zero at lowest lookup-table flow
      ifelse last depth-lookup <= 0.0  ; The last value in lookup table is zero
      [
        set flow-at-wetting 999999
      ]
      [ ; Last value in table is non-zero
        ifelse item (length depth-flow-list - 2) depth-lookup > 0.0 ; The second-to-last value is also non-zero
        [ ; The second-to-last value is also non-zero, so do interpolation
          let d-index-low 0
          while [ item d-index-low depth-lookup <= 0 ] ; Find the first non-zero depth
          [ set d-index-low d-index-low + 1 ]

          let d-index-high d-index-low + 1
          let Q1 item d-index-low depth-flow-list
          let Q2 item d-index-high depth-flow-list
          let D1 item d-index-low depth-lookup
          let D2 item d-index-high depth-lookup

          let slope (D2 - D1) / (Q2 - Q1)
          let y-intercept D2 - (slope * Q2)
          ifelse y-intercept < 0
          [ ; y-intercept is negative, so depth goes to zero before flow does
            set flow-at-wetting (-1 * y-intercept / slope)
          ]
          [ ; y-intercept is positive so cell is extrapolated to have depth > 0 at zero flow.
            set flow-at-wetting 0.0
          ]

          ; Now check whether flow-at-wetting is less than the highest lookup-table flow with non-zero depth
          let Q0 item (d-index-low - 1) depth-flow-list
          if flow-at-wetting < Q0 [ set flow-at-wetting Q0 ]
        ] ; Do interpolation
        [ ; If there is depth only at highest flow, flow-at-wetting is halfway between it and next-lowest flow
          set flow-at-wetting ((last depth-flow-list) + (item (length depth-flow-list - 2) depth-flow-list)) / 2
        ]

      ] ; Last value in table is non-zero
    ] ; Depth is zero at lowest lookup-table flow
    ] ; ask cells

end

to update-habitat ; Observer procedure to set daily flow, temperature, depth, velocity of patches

  ; See the procedure test-hydraulics for a way to test depth and velocity updates

  ; First, get today's flow and temperature
  let prev-flow flow
  set flow ts-get input-time-series time "flow"
  set temperature ts-get input-time-series time "temperature"

  ; Second, interpolate the depth and velocity, and ovi-suitability, from flow
  update-hydraulics-for flow

  ; Finally, re-color the patches
  shade-patches

end

to update-hydraulics-for [a-flow] ; Observer procedure to interpolate depth, velocity of patches from flow

  ; See the procedure test-hydraulics for a way to test depth and velocity updates

    ; First, interpolate the depth index from flow
    let d-index-high 1
    ifelse a-flow < last depth-flow-list
    [
      while [ a-flow > item d-index-high depth-flow-list]
        [ set d-index-high d-index-high + 1 ]
    ]
    [
      set d-index-high (length depth-flow-list - 1)
    ]
    let d-index-low d-index-high - 1

    ; Second, interpolate the velocity index from flow
    let v-index-high 1
    ifelse a-flow < last velocity-flow-list
    [
      while [ a-flow > item v-index-high velocity-flow-list]
        [ set v-index-high v-index-high + 1 ]
    ]
    [
      set v-index-high (length velocity-flow-list - 1)
    ]
    let v-index-low v-index-high - 1

    ;show (word "Flow: " flow " high index: " d-index-high " low index: " d-index-low)
    ;show (word "Flow: " flow " high index: " v-index-high " low index: " v-index-low)

    ; Now have cells do the interpolation
    ; If flow is less than lowest in table,
    ; ->extrapolate depth downward but don't allow neg. depth
    ; ->interpolate velocity between zero and lowest in table
    ask cells
    [
      ; First, see if flow is below the flow at which depth first exceeds zero
      ifelse a-flow <= flow-at-wetting
      [
        set depth 0.0
        set velocity 0.0
      ]
      [
        ; If not, interpolate depth

        let lowQ item d-index-low depth-flow-list
        let highQ item d-index-high depth-flow-list
        let lowD item d-index-low depth-lookup
        let highD item d-index-high depth-lookup
        ; Check for flow-at-wetting and use it for interpolation
        ; if cell is dry at lowQ from interpolation table.
        if lowQ < flow-at-wetting
          [
            set lowQ flow-at-wetting
            if lowD > 0.0 [error "Depth interpolation error: Depth not zero for Q below flow-at-wetting"]
          ]
        let diff highQ - lowQ
        let ratio (a-flow - lowQ) / diff

        set depth lowD + ratio * (highD - lowD)
        if depth < 0 [ set depth 0.0 ]

        ; Then velocity
        ifelse a-flow < (item 0 velocity-flow-list)
        [
          ; Flow is less than lowest in table, so interpolate from zero to lowest table flow
          set velocity (a-flow / first velocity-flow-list) * first velocity-lookup
          ; But velocity must be zero if depth is zero
          if depth <= 0 [set velocity 0.0]
        ]
        [
          set lowQ item v-index-low velocity-flow-list
          set highQ item v-index-high velocity-flow-list
          let lowV item v-index-low velocity-lookup
          let highV item v-index-high velocity-lookup
          ; Check for flow-at-wetting and use it for interpolation
          ; if cell is dry at lowQ from interpolation table.
          if lowQ < flow-at-wetting
            [
              set lowQ flow-at-wetting
              set lowV 0.0 ; Velocity *might* not be zero at flow-at-wetting due to var. in hydraulic sims.
            ]
          set diff highQ - lowQ
          set ratio (a-flow - lowQ) / diff

          set velocity lowV + ratio * (highV - lowV)
          if velocity < 0 [ error (word "Velocity interpolated to negative value at cell " node-number)]
        ]
      ] ; ifelse a-flow < flow-at-wetting

    ]

end

; Observer procedure to color the patches by elevation or depth
to shade-patches
  ; Non-cell patches are given a constant color in read-geom

  ask cells
  [
    ifelse depth <= 0
      [ ; Patch is above water so shade it green
        set pcolor scale-color green (elevation - min-cell-elevation) 0 (max-cell-elevation - min-cell-elevation)
      ]
      [ ; Patch is below water so shade it by depth or velocity
        ; Velocity is shaded yellow-red
        ifelse shade-variable = "depth"
        [ set pcolor scale-color blue depth max-shade-depth  0]
        [ set pcolor (list 255 (255 * (1 - (min (list velocity max-shade-velocity) / max-shade-velocity))) 0)]

      ]
  ]

end

to test-cell-hydraulics  ; A test procedure to output hydraulic data for selected cells

  ; Set the range of flows (m3/s) and increment
  let min-flow 0.1
  let increment 0.1
  let max-flow 70

  let this-flow min-flow

  ; Set up an output file
   file-close ; Just in case one is open.
   if file-exists? "Hydraulic-Cell-Test-Output.csv" [ file-delete "Hydraulic-Cell-Test-Output.csv" ]
   file-open "Hydraulic-Cell-Test-Output.csv"
   file-print (word "Digital river model individual cell hydraulic test output file, Created " date-and-time)
   file-print "Cell,flow-at-wetting,Flow,Depth,Velocity"

  ; Select some cells
  let the-test-cells n-of 20 cells

  ; Update hydraulics and write output
  while [this-flow <= max-flow]
  [
    update-hydraulics-for this-flow

    ; New write output
    ask the-test-cells
    [
      file-print (word node-number "," flow-at-wetting "," this-flow "," depth "," velocity)
    ]

    ; And remember to increment the flow!
    set this-flow this-flow + increment
    show (word "Processing flow: " this-flow)

  ]   ; while

  ; Now tidy up
  file-close

end

;; NOTE: the following ts- procedures are copied from the file time-series.nls distributed
;; with the Time extension for NetLogo 6.1.1
;; They should not be modified.

;; Implementation of what were primitives from the time-extension time-series tool
to-report ts-create [col-name-list]
  report  (list sentence "LOGOTIME" col-name-list)
end

to-report ts-add-row [ltime-series row]

  (ifelse
    length ltime-series = 1 or time:is-after? (__time-of row) (__time-of last ltime-series) [
      report lput (__row-copy row) ltime-series
    ]
    time:is-before? (__time-of row) (__time-of item 1 ltime-series) [
      report insert-item 1 ltime-series (__row-copy row)
    ]
    [ ;else
      let b&a __indices-before-and-after ltime-series (__time-of row)
      report insert-item (last b&a) ltime-series (__row-copy row)
    ]
  )
end

to-report __row-copy [row]
  report fput (time:copy first row) (butfirst row)
end

to-report ts-get [ltime-series logotime col-name]
  let b&a ts-get-before-and-after ltime-series logotime
  let columns (item 0 ltime-series)
  (ifelse
    first b&a = nobody [report __col-of-row columns (last b&a) col-name] ; need this explicitly so we don't try to get __time-of first b&a iin the next conditional when first b&a is nobody
    last b&a = nobody or
    (abs(time:difference-between logotime (__time-of first b&a) "seconds") <= abs(time:difference-between logotime (__time-of last b&a) "seconds"))[
      report __col-of-row columns (first b&a) col-name   ; logotime is closer to the item before than after
    ]  [ ; else
    report __col-of-row columns (last b&a) col-name
    ]
  )
end

to-report ts-get-before-and-after [ltime-series logotime]
  let b&a __indices-before-and-after ltime-series logotime
  let before ifelse-value (first b&a = nobody) [nobody] [item (first b&a) ltime-series]
  let after ifelse-value (last b&a = nobody) [nobody] [item (last b&a) ltime-series]
  report list before after
end

to-report ts-get-interp [ltime-series logotime col-name]
  ;; if col-name is a specific column, return just the linearly interpolated value
  ;; if col-name is "all" then return a whole interpolated row, including an interpolated logotime

  let before-and-after ts-get-before-and-after ltime-series logotime
  let col-names first ltime-series

  ifelse col-name = "all" or col-name = "ALL" [
    report fput logotime map [col -> __interp before-and-after logotime col-names col] butfirst col-names
  ] [
    report __interp before-and-after logotime col-names col-name
  ]
end

to-report ts-get-exact [ltime-series logotime col-name]
  let closest-row ts-get ltime-series logotime "all"
  ifelse time:is-equal? (item 0 closest-row) logotime [
    report __col-of-row (item 0 ltime-series) closest-row col-name
  ] [
    error word "There is no row in this timeseries with exactly the time " logotime
  ]

end

to-report ts-get-range [ltime-series logotime1 logotime2 col-name]
  ; reports a subset of the time-series between the two logotimes.

; reverse the times if logotime2 is before logotime1
  if time:is-before? logotime2 logotime1 [
    let temp1 logotime1
    set logotime1 logotime2
    set logotime2 temp1
  ]

  let max-index (length ltime-series)

  ; get the start index of the range
  let b&a-t1 __indices-before-and-after ltime-series logotime1
  let start-index nobody
  ifelse first b&a-t1 = last b&a-t1 [ ; if b&a are the same, there was an exact match. So, need to get the index of the first row with that time
    set start-index first b&a-t1
    while [start-index > 1 and time:is-equal? logotime1 (__time-of item (start-index - 1) ltime-series)] [set start-index start-index - 1]
  ] [ ; if there isn't an exact match, then the after index will be the start of the returned range.
    set start-index __after-index (last b&a-t1) max-index
  ]

  ; get the end index of the range
  let b&a-t2 __indices-before-and-after ltime-series logotime2
  let end-index nobody
  ifelse (first b&a-t2 = last b&a-t2) [ ; if b&a are the same, there was an exact match. So need to get the index of the first row after that time.
    set end-index last b&a-t2
    while [end-index < max-index and time:is-equal? logotime2 (__time-of item end-index ltime-series)] [set end-index end-index + 1]
  ] [ ; if there isn't an exact match, then the after index will be the end of the returned range
    set end-index __after-index (last b&a-t2) max-index
  ]

  ; report the columns of the sublist between aftere-t1 and after-t2
  let columns (item 0 ltime-series)
  report map [row -> __col-of-row columns row col-name] sublist ltime-series start-index end-index
end

to-report ts-load [filepath]
  report ts-load-with-format filepath nobody
end

to-report ts-load-with-format [filepath format-string]
  let temp-ts __remove-comment-lines (csv:from-file filepath ",")

  let columns fput "LOGOTIME" butfirst item 0 temp-ts ; replace the first column with "LOGOTIME"

  report fput columns __sort-by-date map [ row ->
    __convert-row-with-format row format-string
  ] (butfirst temp-ts)
end

to-report ts-has-repeat-times? [ltime-series]
  foreach (range 1 (length ltime-series - 1)) [i ->
    if time:is-equal? (__time-of item i ltime-series) (__time-of item (i + 1) ltime-series) [report true]]
  report false
end

to-report ts-has-repeat-of-time? [ltime-series logotime]
  let b&a __indices-before-and-after ltime-series logotime
  let i1 first b&a
  let i2 last b&a
  ; if the indices are the same, that just means there was an exact match -> still need to check above and below
  ifelse i1 = i2 [
    (ifelse
      i1 > 1 and time:is-equal? (__time-of item (i1 - 1) ltime-series) (__time-of item i1 ltime-series) [
        report true
      ]
      i2 < (length ltime-series - 1) and time:is-equal? (__time-of item i2 ltime-series) (__time-of item (i2 + 1) ltime-series) [
        report true
      ]
      [report false]
    )
  ] [
    ; if the i1 != i2, there isn't even an exact match, let alone a repeat.
    report false
  ]
end

to ts-write [ltime-series filepath]
  csv:to-file filepath fput (first ltime-series) map [row ->
    fput (__show-default (first row)) (butfirst row)]
    butfirst ltime-series
end

;**********************************
;********Helper Procedures for ts- procedures *********
;**********************************
to-report __interp [before-and-after logotime col-names col-name]
  let before-val __col-of-row col-names (item 0 before-and-after) col-name
  let after-val __col-of-row col-names (item 1 before-and-after) col-name

  (ifelse
    before-val = after-val [
      report before-val]
    before-val = nobody [
      report after-val]
    after-val = nobody [
      report before-val]
    [ ; else
      let before-time item 0 item 0 before-and-after
      let after-time item 0 item 1 before-and-after
      let before-after-time-diff (time:difference-between before-time after-time "seconds")
      let slope (after-val - before-val) / before-after-time-diff
      let before-logotime-diff (time:difference-between before-time logotime "seconds")

      report before-val + slope * before-logotime-diff
    ]
  )
end

to-report __indices-before-and-after [timeseries logotime]
  ; return the indexes of the rows in timeseries that have logotimes above and below the input logotime.
  ; if there is an exact match, return the index of the exact row twice.
  ; the column names are included so that you can just use the index on the timeseries list and not butfirst timeseries list.
  ; This is equivalent to treating ltime-series objects here as starting at index 1
  let i round ((length timeseries) / 2)
  let max-i length timeseries - 1
  let min-i 1
  while [true] [
    (ifelse
      time:is-equal? logotime (__time-of item i timeseries) [report list i i]
      (time:is-before? logotime (__time-of item i timeseries)) and i = 1 [report list nobody i]
      (time:is-after? logotime (__time-of item i timeseries)) and i = (length timeseries - 1) [report list i nobody]
      (time:is-after? logotime (__time-of item i timeseries)) and (time:is-before? logotime (__time-of item (i + 1) timeseries)) [report list i (i + 1)]
      time:is-before? logotime (__time-of item i timeseries) [
        set max-i i
        set i floor ((i + min-i) / 2)
      ]
      __is-after-or-equal? logotime (__time-of item (i + 1) timeseries) [
        set min-i i
        set i ceiling ((i + max-i) / 2)
      ]
    )
  ]
end

to-report __time-of [row]
  report item 0 row
end

to-report __sort-by-date [timeseries]
  report sort-by [[row1 row2] -> time:is-before? (item 0 row1) (item 0 row2)] timeseries
end

to-report __remove-comment-lines [data]
  while [(first (first (first data)) = ";")]  ;; remove lines that start with a semicolon
      [set data but-first data]
  report data
end

to-report __convert-row-with-format [row format-string] ; helper
  let new-time ifelse-value format-string = nobody [time:create item 0 row] [time:create-with-format (item 0 row) format-string]
  ;report fput new-time (map [i -> ifelse-value is-string? i [read-from-string i] [i]] butfirst row)
  report fput new-time butfirst row
end

to-report __convert-row [row] ; helper
  report __convert-row-with-format row nobody
end

to-report __col-of-row [columns row col-name]
  (ifelse
    col-name = "all" or col-name = "ALL" [report row]
    col-name = "LOGOTIME" or col-name = "logotime" [report item 0 row]
    [ ;else
      let pos position col-name columns
      if pos = false [error (word "'" col-name "' is not a column in this time series")]
      report item pos row
    ]
  )
end

to-report __is-before-or-equal? [logotime1 logotime2]
  report time:is-before? logotime1 logotime2 or time:is-equal? logotime1 logotime2
end

to-report __is-after-or-equal? [logotime1 logotime2]
  report time:is-after? logotime1 logotime2 or time:is-equal? logotime1 logotime2
end

to-report __logotime-variety [logotime]
  let variety nobody
  carefully [
    let temp time:is-equal? logotime time:create "2000-12-01 1"
    set variety "DATETIME"
  ] [
    carefully [
      let temp time:is-equal? logotime time:create "2000-12-01"
      set variety "DATE"
    ] [
      let temp time:is-equal? logotime time:create "12-01"
      set variety "DAY"
    ]
  ]
  report variety
end

to-report __show-default [logotime]
  ; Like time:show, but it shows the default format depending on the variety
  ; of the logotime .
  let variety __logotime-variety logotime
  (ifelse
    variety = "DATETIME" [report time:show logotime "yyyy-MM-dd HH:mm:ss.SSS"]
    variety = "DATE" [report time:show logotime "yyyy-MM-dd"]
    variety = "DAY" [report time:show logotime "MM-dd"]
  )
end

to-report __after-index [after-i max-index]
  ; if the after-index exists, report it, otherwise report the max-index
  report ifelse-value after-i = nobody [max-index] [after-i]
end

;; END OF TIME-SERIES PROCEDURES
@#$#@#$#@
GRAPHICS-WINDOW
181
10
639
963
-1
-1
0.5
1
10
1
1
1
0
0
0
1
0
457
0
943
1
1
1
ticks
30.0

BUTTON
4
12
96
45
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
3
109
145
169
start-time
4/1/2010 00:00
1
0
String

BUTTON
4
63
67
96
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

OUTPUT
17
983
637
1067
12

BUTTON
74
63
137
96
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
4
179
96
224
shade-variable
shade-variable
"depth" "velocity"
0

SWITCH
4
232
119
265
file-output?
file-output?
1
1
-1000

@#$#@#$#@
# Digital River Model

This code provides a reusable simulation of river hydraulics and habitat. See the accompanying unpublished report "Digital River Documentation".

The code is adapted from the habitat modeling methods of the FYFAM frog model, available at: https://ecomodel.humboldt.edu/foothill-yellow-legged-frog-assessment-model

and published as supplemental materials to:

Railsback, S. F., B. C. Harvey, S. J. Kupferberg, M. M. Lang, S. McBain, and H. H. J. Welsh. 2016. Modeling potential river management conflicts between frogs and salmonids. Canadian Journal of Fisheries and Aquatic Sciences 73:773-784.

This code represents space as square grid cells. For an alternative using irregular polygon cells, see the InSTREAM trout model at:

https://ecomodel.humboldt.edu/instream-7-and-insalmo-7


## Copying

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>.

## GPL license
                    GNU GENERAL PUBLIC LICENSE
                       Version 3, 29 June 2007

 Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

                            Preamble

  The GNU General Public License is a free, copyleft license for
software and other kinds of works.

  The licenses for most software and other practical works are designed
to take away your freedom to share and change the works.  By contrast,
the GNU General Public License is intended to guarantee your freedom to
share and change all versions of a program--to make sure it remains free
software for all its users.  We, the Free Software Foundation, use the
GNU General Public License for most of our software; it applies also to
any other work released this way by its authors.  You can apply it to
your programs, too.

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
them if you wish), that you receive source code or can get it if you
want it, that you can change the software or use pieces of it in new
free programs, and that you know you can do these things.

  To protect your rights, we need to prevent others from denying you
these rights or asking you to surrender the rights.  Therefore, you have
certain responsibilities if you distribute copies of the software, or if
you modify it: responsibilities to respect the freedom of others.

  For example, if you distribute copies of such a program, whether
gratis or for a fee, you must pass on to the recipients the same
freedoms that you received.  You must make sure that they, too, receive
or can get the source code.  And you must show them these terms so they
know their rights.

  Developers that use the GNU GPL protect your rights with two steps:
(1) assert copyright on the software, and (2) offer you this License
giving you legal permission to copy, distribute and/or modify it.

  For the developers' and authors' protection, the GPL clearly explains
that there is no warranty for this free software.  For both users' and
authors' sake, the GPL requires that modified versions be marked as
changed, so that their problems will not be attributed erroneously to
authors of previous versions.

  Some devices are designed to deny users access to install or run
modified versions of the software inside them, although the manufacturer
can do so.  This is fundamentally incompatible with the aim of
protecting users' freedom to change the software.  The systematic
pattern of such abuse occurs in the area of products for individuals to
use, which is precisely where it is most unacceptable.  Therefore, we
have designed this version of the GPL to prohibit the practice for those
products.  If such problems arise substantially in other domains, we
stand ready to extend this provision to those domains in future versions
of the GPL, as needed to protect the freedom of users.

  Finally, every program is threatened constantly by software patents.
States should not allow patents to restrict development and use of
software on general-purpose computers, but in those that do, we wish to
avoid the special danger that patents applied to a free program could
make it effectively proprietary.  To prevent this, the GPL assures that
patents cannot be used to render the program non-free.

  The precise terms and conditions for copying, distribution and
modification follow.

                       TERMS AND CONDITIONS

  \0. Definitions.

  "This License" refers to version 3 of the GNU General Public License.

  "Copyright" also means copyright-like laws that apply to other kinds of
works, such as semiconductor masks.

  "The Program" refers to any copyrightable work licensed under this
License.  Each licensee is addressed as "you".  "Licensees" and
"recipients" may be individuals or organizations.

  To "modify" a work means to copy from or adapt all or part of the work
in a fashion requiring copyright permission, other than the making of an
exact copy.  The resulting work is called a "modified version" of the
earlier work or a work "based on" the earlier work.

  A "covered work" means either the unmodified Program or a work based
on the Program.

  To "propagate" a work means to do anything with it that, without
permission, would make you directly or secondarily liable for
infringement under applicable copyright law, except executing it on a
computer or modifying a private copy.  Propagation includes copying,
distribution (with or without modification), making available to the
public, and in some countries other activities as well.

  To "convey" a work means any kind of propagation that enables other
parties to make or receive copies.  Mere interaction with a user through
a computer network, with no transfer of a copy, is not conveying.

  An interactive user interface displays "Appropriate Legal Notices"
to the extent that it includes a convenient and prominently visible
feature that (1) displays an appropriate copyright notice, and (2)
tells the user that there is no warranty for the work (except to the
extent that warranties are provided), that licensees may convey the
work under this License, and how to view a copy of this License.  If
the interface presents a list of user commands or options, such as a
menu, a prominent item in the list meets this criterion.

  \1. Source Code.

  The "source code" for a work means the preferred form of the work
for making modifications to it.  "Object code" means any non-source
form of a work.

  A "Standard Interface" means an interface that either is an official
standard defined by a recognized standards body, or, in the case of
interfaces specified for a particular programming language, one that
is widely used among developers working in that language.

  The "System Libraries" of an executable work include anything, other
than the work as a whole, that (a) is included in the normal form of
packaging a Major Component, but which is not part of that Major
Component, and (b) serves only to enable use of the work with that
Major Component, or to implement a Standard Interface for which an
implementation is available to the public in source code form.  A
"Major Component", in this context, means a major essential component
(kernel, window system, and so on) of the specific operating system
(if any) on which the executable work runs, or a compiler used to
produce the work, or an object code interpreter used to run it.

  The "Corresponding Source" for a work in object code form means all
the source code needed to generate, install, and (for an executable
work) run the object code and to modify the work, including scripts to
control those activities.  However, it does not include the work's
System Libraries, or general-purpose tools or generally available free
programs which are used unmodified in performing those activities but
which are not part of the work.  For example, Corresponding Source
includes interface definition files associated with source files for
the work, and the source code for shared libraries and dynamically
linked subprograms that the work is specifically designed to require,
such as by intimate data communication or control flow between those
subprograms and other parts of the work.

  The Corresponding Source need not include anything that users
can regenerate automatically from other parts of the Corresponding
Source.

  The Corresponding Source for a work in source code form is that
same work.

  \2. Basic Permissions.

  All rights granted under this License are granted for the term of
copyright on the Program, and are irrevocable provided the stated
conditions are met.  This License explicitly affirms your unlimited
permission to run the unmodified Program.  The output from running a
covered work is covered by this License only if the output, given its
content, constitutes a covered work.  This License acknowledges your
rights of fair use or other equivalent, as provided by copyright law.

  You may make, run and propagate covered works that you do not
convey, without conditions so long as your license otherwise remains
in force.  You may convey covered works to others for the sole purpose
of having them make modifications exclusively for you, or provide you
with facilities for running those works, provided that you comply with
the terms of this License in conveying all material for which you do
not control copyright.  Those thus making or running the covered works
for you must do so exclusively on your behalf, under your direction
and control, on terms that prohibit them from making any copies of
your copyrighted material outside their relationship with you.

  Conveying under any other circumstances is permitted solely under
the conditions stated below.  Sublicensing is not allowed; section 10
makes it unnecessary.

  \3. Protecting Users' Legal Rights From Anti-Circumvention Law.

  No covered work shall be deemed part of an effective technological
measure under any applicable law fulfilling obligations under article
11 of the WIPO copyright treaty adopted on 20 December 1996, or
similar laws prohibiting or restricting circumvention of such
measures.

  When you convey a covered work, you waive any legal power to forbid
circumvention of technological measures to the extent such circumvention
is effected by exercising rights under this License with respect to
the covered work, and you disclaim any intention to limit operation or
modification of the work as a means of enforcing, against the work's
users, your or third parties' legal rights to forbid circumvention of
technological measures.

  \4. Conveying Verbatim Copies.

  You may convey verbatim copies of the Program's source code as you
receive it, in any medium, provided that you conspicuously and
appropriately publish on each copy an appropriate copyright notice;
keep intact all notices stating that this License and any
non-permissive terms added in accord with section 7 apply to the code;
keep intact all notices of the absence of any warranty; and give all
recipients a copy of this License along with the Program.

  You may charge any price or no price for each copy that you convey,
and you may offer support or warranty protection for a fee.

  \5. Conveying Modified Source Versions.

  You may convey a work based on the Program, or the modifications to
produce it from the Program, in the form of source code under the
terms of section 4, provided that you also meet all of these conditions:

    a) The work must carry prominent notices stating that you modified
    it, and giving a relevant date.

    b) The work must carry prominent notices stating that it is
    released under this License and any conditions added under section
    7.  This requirement modifies the requirement in section 4 to
    "keep intact all notices".

    c) You must license the entire work, as a whole, under this
    License to anyone who comes into possession of a copy.  This
    License will therefore apply, along with any applicable section 7
    additional terms, to the whole of the work, and all its parts,
    regardless of how they are packaged.  This License gives no
    permission to license the work in any other way, but it does not
    invalidate such permission if you have separately received it.

    d) If the work has interactive user interfaces, each must display
    Appropriate Legal Notices; however, if the Program has interactive
    interfaces that do not display Appropriate Legal Notices, your
    work need not make them do so.

  A compilation of a covered work with other separate and independent
works, which are not by their nature extensions of the covered work,
and which are not combined with it such as to form a larger program,
in or on a volume of a storage or distribution medium, is called an
"aggregate" if the compilation and its resulting copyright are not
used to limit the access or legal rights of the compilation's users
beyond what the individual works permit.  Inclusion of a covered work
in an aggregate does not cause this License to apply to the other
parts of the aggregate.

  \6. Conveying Non-Source Forms.

  You may convey a covered work in object code form under the terms
of sections 4 and 5, provided that you also convey the
machine-readable Corresponding Source under the terms of this License,
in one of these ways:

    a) Convey the object code in, or embodied in, a physical product
    (including a physical distribution medium), accompanied by the
    Corresponding Source fixed on a durable physical medium
    customarily used for software interchange.

    b) Convey the object code in, or embodied in, a physical product
    (including a physical distribution medium), accompanied by a
    written offer, valid for at least three years and valid for as
    long as you offer spare parts or customer support for that product
    model, to give anyone who possesses the object code either (1) a
    copy of the Corresponding Source for all the software in the
    product that is covered by this License, on a durable physical
    medium customarily used for software interchange, for a price no
    more than your reasonable cost of physically performing this
    conveying of source, or (2) access to copy the
    Corresponding Source from a network server at no charge.

    c) Convey individual copies of the object code with a copy of the
    written offer to provide the Corresponding Source.  This
    alternative is allowed only occasionally and noncommercially, and
    only if you received the object code with such an offer, in accord
    with subsection 6b.

    d) Convey the object code by offering access from a designated
    place (gratis or for a charge), and offer equivalent access to the
    Corresponding Source in the same way through the same place at no
    further charge.  You need not require recipients to copy the
    Corresponding Source along with the object code.  If the place to
    copy the object code is a network server, the Corresponding Source
    may be on a different server (operated by you or a third party)
    that supports equivalent copying facilities, provided you maintain
    clear directions next to the object code saying where to find the
    Corresponding Source.  Regardless of what server hosts the
    Corresponding Source, you remain obligated to ensure that it is
    available for as long as needed to satisfy these requirements.

    e) Convey the object code using peer-to-peer transmission, provided
    you inform other peers where the object code and Corresponding
    Source of the work are being offered to the general public at no
    charge under subsection 6d.

  A separable portion of the object code, whose source code is excluded
from the Corresponding Source as a System Library, need not be
included in conveying the object code work.

  A "User Product" is either (1) a "consumer product", which means any
tangible personal property which is normally used for personal, family,
or household purposes, or (2) anything designed or sold for incorporation
into a dwelling.  In determining whether a product is a consumer product,
doubtful cases shall be resolved in favor of coverage.  For a particular
product received by a particular user, "normally used" refers to a
typical or common use of that class of product, regardless of the status
of the particular user or of the way in which the particular user
actually uses, or expects or is expected to use, the product.  A product
is a consumer product regardless of whether the product has substantial
commercial, industrial or non-consumer uses, unless such uses represent
the only significant mode of use of the product.

  "Installation Information" for a User Product means any methods,
procedures, authorization keys, or other information required to install
and execute modified versions of a covered work in that User Product from
a modified version of its Corresponding Source.  The information must
suffice to ensure that the continued functioning of the modified object
code is in no case prevented or interfered with solely because
modification has been made.

  If you convey an object code work under this section in, or with, or
specifically for use in, a User Product, and the conveying occurs as
part of a transaction in which the right of possession and use of the
User Product is transferred to the recipient in perpetuity or for a
fixed term (regardless of how the transaction is characterized), the
Corresponding Source conveyed under this section must be accompanied
by the Installation Information.  But this requirement does not apply
if neither you nor any third party retains the ability to install
modified object code on the User Product (for example, the work has
been installed in ROM).

  The requirement to provide Installation Information does not include a
requirement to continue to provide support service, warranty, or updates
for a work that has been modified or installed by the recipient, or for
the User Product in which it has been modified or installed.  Access to a
network may be denied when the modification itself materially and
adversely affects the operation of the network or violates the rules and
protocols for communication across the network.

  Corresponding Source conveyed, and Installation Information provided,
in accord with this section must be in a format that is publicly
documented (and with an implementation available to the public in
source code form), and must require no special password or key for
unpacking, reading or copying.

  \7. Additional Terms.

  "Additional permissions" are terms that supplement the terms of this
License by making exceptions from one or more of its conditions.
Additional permissions that are applicable to the entire Program shall
be treated as though they were included in this License, to the extent
that they are valid under applicable law.  If additional permissions
apply only to part of the Program, that part may be used separately
under those permissions, but the entire Program remains governed by
this License without regard to the additional permissions.

  When you convey a copy of a covered work, you may at your option
remove any additional permissions from that copy, or from any part of
it.  (Additional permissions may be written to require their own
removal in certain cases when you modify the work.)  You may place
additional permissions on material, added by you to a covered work,
for which you have or can give appropriate copyright permission.

  Notwithstanding any other provision of this License, for material you
add to a covered work, you may (if authorized by the copyright holders of
that material) supplement the terms of this License with terms:

    a) Disclaiming warranty or limiting liability differently from the
    terms of sections 15 and 16 of this License; or

    b) Requiring preservation of specified reasonable legal notices or
    author attributions in that material or in the Appropriate Legal
    Notices displayed by works containing it; or

    c) Prohibiting misrepresentation of the origin of that material, or
    requiring that modified versions of such material be marked in
    reasonable ways as different from the original version; or

    d) Limiting the use for publicity purposes of names of licensors or
    authors of the material; or

    e) Declining to grant rights under trademark law for use of some
    trade names, trademarks, or service marks; or

    f) Requiring indemnification of licensors and authors of that
    material by anyone who conveys the material (or modified versions of
    it) with contractual assumptions of liability to the recipient, for
    any liability that these contractual assumptions directly impose on
    those licensors and authors.

  All other non-permissive additional terms are considered "further
restrictions" within the meaning of section 10.  If the Program as you
received it, or any part of it, contains a notice stating that it is
governed by this License along with a term that is a further
restriction, you may remove that term.  If a license document contains
a further restriction but permits relicensing or conveying under this
License, you may add to a covered work material governed by the terms
of that license document, provided that the further restriction does
not survive such relicensing or conveying.

  If you add terms to a covered work in accord with this section, you
must place, in the relevant source files, a statement of the
additional terms that apply to those files, or a notice indicating
where to find the applicable terms.

  Additional terms, permissive or non-permissive, may be stated in the
form of a separately written license, or stated as exceptions;
the above requirements apply either way.

  \8. Termination.

  You may not propagate or modify a covered work except as expressly
provided under this License.  Any attempt otherwise to propagate or
modify it is void, and will automatically terminate your rights under
this License (including any patent licenses granted under the third
paragraph of section 11).

  However, if you cease all violation of this License, then your
license from a particular copyright holder is reinstated (a)
provisionally, unless and until the copyright holder explicitly and
finally terminates your license, and (b) permanently, if the copyright
holder fails to notify you of the violation by some reasonable means
prior to 60 days after the cessation.

  Moreover, your license from a particular copyright holder is
reinstated permanently if the copyright holder notifies you of the
violation by some reasonable means, this is the first time you have
received notice of violation of this License (for any work) from that
copyright holder, and you cure the violation prior to 30 days after
your receipt of the notice.

  Termination of your rights under this section does not terminate the
licenses of parties who have received copies or rights from you under
this License.  If your rights have been terminated and not permanently
reinstated, you do not qualify to receive new licenses for the same
material under section 10.

  \9. Acceptance Not Required for Having Copies.

  You are not required to accept this License in order to receive or
run a copy of the Program.  Ancillary propagation of a covered work
occurring solely as a consequence of using peer-to-peer transmission
to receive a copy likewise does not require acceptance.  However,
nothing other than this License grants you permission to propagate or
modify any covered work.  These actions infringe copyright if you do
not accept this License.  Therefore, by modifying or propagating a
covered work, you indicate your acceptance of this License to do so.

  \10. Automatic Licensing of Downstream Recipients.

  Each time you convey a covered work, the recipient automatically
receives a license from the original licensors, to run, modify and
propagate that work, subject to this License.  You are not responsible
for enforcing compliance by third parties with this License.

  An "entity transaction" is a transaction transferring control of an
organization, or substantially all assets of one, or subdividing an
organization, or merging organizations.  If propagation of a covered
work results from an entity transaction, each party to that
transaction who receives a copy of the work also receives whatever
licenses to the work the party's predecessor in interest had or could
give under the previous paragraph, plus a right to possession of the
Corresponding Source of the work from the predecessor in interest, if
the predecessor has it or can get it with reasonable efforts.

  You may not impose any further restrictions on the exercise of the
rights granted or affirmed under this License.  For example, you may
not impose a license fee, royalty, or other charge for exercise of
rights granted under this License, and you may not initiate litigation
(including a cross-claim or counterclaim in a lawsuit) alleging that
any patent claim is infringed by making, using, selling, offering for
sale, or importing the Program or any portion of it.

  \11. Patents.

  A "contributor" is a copyright holder who authorizes use under this
License of the Program or a work on which the Program is based.  The
work thus licensed is called the contributor's "contributor version".

  A contributor's "essential patent claims" are all patent claims
owned or controlled by the contributor, whether already acquired or
hereafter acquired, that would be infringed by some manner, permitted
by this License, of making, using, or selling its contributor version,
but do not include claims that would be infringed only as a
consequence of further modification of the contributor version.  For
purposes of this definition, "control" includes the right to grant
patent sublicenses in a manner consistent with the requirements of
this License.

  Each contributor grants you a non-exclusive, worldwide, royalty-free
patent license under the contributor's essential patent claims, to
make, use, sell, offer for sale, import and otherwise run, modify and
propagate the contents of its contributor version.

  In the following three paragraphs, a "patent license" is any express
agreement or commitment, however denominated, not to enforce a patent
(such as an express permission to practice a patent or covenant not to
sue for patent infringement).  To "grant" such a patent license to a
party means to make such an agreement or commitment not to enforce a
patent against the party.

  If you convey a covered work, knowingly relying on a patent license,
and the Corresponding Source of the work is not available for anyone
to copy, free of charge and under the terms of this License, through a
publicly available network server or other readily accessible means,
then you must either (1) cause the Corresponding Source to be so
available, or (2) arrange to deprive yourself of the benefit of the
patent license for this particular work, or (3) arrange, in a manner
consistent with the requirements of this License, to extend the patent
license to downstream recipients.  "Knowingly relying" means you have
actual knowledge that, but for the patent license, your conveying the
covered work in a country, or your recipient's use of the covered work
in a country, would infringe one or more identifiable patents in that
country that you have reason to believe are valid.

  If, pursuant to or in connection with a single transaction or
arrangement, you convey, or propagate by procuring conveyance of, a
covered work, and grant a patent license to some of the parties
receiving the covered work authorizing them to use, propagate, modify
or convey a specific copy of the covered work, then the patent license
you grant is automatically extended to all recipients of the covered
work and works based on it.

  A patent license is "discriminatory" if it does not include within
the scope of its coverage, prohibits the exercise of, or is
conditioned on the non-exercise of one or more of the rights that are
specifically granted under this License.  You may not convey a covered
work if you are a party to an arrangement with a third party that is
in the business of distributing software, under which you make payment
to the third party based on the extent of your activity of conveying
the work, and under which the third party grants, to any of the
parties who would receive the covered work from you, a discriminatory
patent license (a) in connection with copies of the covered work
conveyed by you (or copies made from those copies), or (b) primarily
for and in connection with specific products or compilations that
contain the covered work, unless you entered into that arrangement,
or that patent license was granted, prior to 28 March 2007.

  Nothing in this License shall be construed as excluding or limiting
any implied license or other defenses to infringement that may
otherwise be available to you under applicable patent law.

  \12. No Surrender of Others' Freedom.

  If conditions are imposed on you (whether by court order, agreement or
otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot convey a
covered work so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you may
not convey it at all.  For example, if you agree to terms that obligate you
to collect a royalty for further conveying from those to whom you convey
the Program, the only way you could satisfy both those terms and this
License would be to refrain entirely from conveying the Program.

  \13. Use with the GNU Affero General Public License.

  Notwithstanding any other provision of this License, you have
permission to link or combine any covered work with a work licensed
under version 3 of the GNU Affero General Public License into a single
combined work, and to convey the resulting work.  The terms of this
License will continue to apply to the part which is the covered work,
but the special requirements of the GNU Affero General Public License,
section 13, concerning interaction through a network will apply to the
combination as such.

  \14. Revised Versions of this License.

  The Free Software Foundation may publish revised and/or new versions of
the GNU General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.

  Each version is given a distinguishing version number.  If the
Program specifies that a certain numbered version of the GNU General
Public License "or any later version" applies to it, you have the
option of following the terms and conditions either of that numbered
version or of any later version published by the Free Software
Foundation.  If the Program does not specify a version number of the
GNU General Public License, you may choose any version ever published
by the Free Software Foundation.

  If the Program specifies that a proxy can decide which future
versions of the GNU General Public License can be used, that proxy's
public statement of acceptance of a version permanently authorizes you
to choose that version for the Program.

  Later license versions may give you additional or different
permissions.  However, no additional obligations are imposed on any
author or copyright holder as a result of your choosing to follow a
later version.

  \15. Disclaimer of Warranty.

  THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

  \16. Limitation of Liability.

  IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

  \17. Interpretation of Sections 15 and 16.

  If the disclaimer of warranty and limitation of liability provided
above cannot be given local legal effect according to their terms,
reviewing courts shall apply local law that most closely approximates
an absolute waiver of all civil liability in connection with the
Program, unless a warranty or assumption of liability accompanies a
copy of the Program in return for a fee.

                     END OF TERMS AND CONDITIONS

            How to Apply These Terms to Your New Programs

  If you develop a new program, and you want it to be of the greatest
possible use to the public, the best way to achieve this is to make it
free software which everyone can redistribute and change under these terms.

  To do so, attach the following notices to the program.  It is safest
to attach them to the start of each source file to most effectively
state the exclusion of warranty; and each file should have at least
the "copyright" line and a pointer to where the full notice is found.

    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Also add information on how to contact you by electronic and paper mail.

  If the program does terminal interaction, make it output a short
notice like this when it starts in an interactive mode:

    <program>  Copyright (C) <year>  <name of author>
    This program comes with ABSOLUTELY NO WARRANTY; for details type `show w'.
    This is free software, and you are welcome to redistribute it
    under certain conditions; type `show c' for details.

The hypothetical commands `show w' and `show c' should show the appropriate
parts of the General Public License.  Of course, your program's commands
might be different; for a GUI interface, you would use an "about box".

  You should also get your employer (if you work as a programmer) or school,
if any, to sign a "copyright disclaimer" for the program, if necessary.
For more information on this, and how to apply and follow the GNU GPL, see
<http://www.gnu.org/licenses/>.

  The GNU General Public License does not permit incorporating your program
into proprietary programs.  If your program is a subroutine library, you
may consider it more useful to permit linking proprietary applications with
the library.  If this is what you want to do, use the GNU Lesser General
Public License instead of this License.  But first, please read
<http://www.gnu.org/philosophy/why-not-lgpl.html>.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

frog top
true
0
Polygon -7500403 true true 146 18 135 30 119 42 105 90 90 150 105 195 135 225 165 225 195 195 210 150 195 90 180 41 165 30 155 18
Polygon -7500403 true true 91 176 67 148 70 121 66 119 61 133 59 111 53 111 52 131 47 115 42 120 46 146 55 187 80 237 106 269 116 268 114 214 131 222
Polygon -7500403 true true 185 62 234 84 223 51 226 48 234 61 235 38 240 38 243 60 252 46 255 49 244 95 188 92
Polygon -7500403 true true 115 62 66 84 77 51 74 48 66 61 65 38 60 38 57 60 48 46 45 49 56 95 112 92
Polygon -7500403 true true 200 186 233 148 230 121 234 119 239 133 241 111 247 111 248 131 253 115 258 120 254 146 245 187 220 237 194 269 184 268 186 214 169 222
Circle -16777216 true false 157 38 18
Circle -16777216 true false 125 38 18

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.3.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="ExampleExperiment" repetitions="1" runMetricsEveryStep="true">
    <setup>reset</setup>
    <go>go</go>
    <metric>formatted-time</metric>
    <metric>flow</metric>
    <metric>temperature</metric>
    <metric>count breeders</metric>
    <metric>count eggmasses</metric>
    <metric>count tadpoles</metric>
    <metric>number-of-new-frogs</metric>
    <metric>eggmasses-created</metric>
    <metric>eggmasses-died-desic</metric>
    <metric>eggmasses-died-scour</metric>
    <metric>eggmasses-emptied</metric>
    <metric>median-metamorph-date</metric>
    <metric>tadpoles-hatched</metric>
    <metric>tadpoles-died-desic</metric>
    <metric>tadpoles-died-scour</metric>
    <enumeratedValueSet variable="file-output?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="start-time">
      <value value="&quot;4/1/2009 00:00&quot;"/>
      <value value="&quot;4/1/2010 00:00&quot;"/>
      <value value="&quot;4/1/2011 00:00&quot;"/>
      <value value="&quot;4/1/2012 00:00&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
